#!/bin/bash
set -e
exec 2>&1

# add stdout to report
echo '<H2>Job stdout/stderr</H2>'
echo '<PRE>'

# load evironment from cmd line
export -- "$@" dummy

# avoid using ~/.same
export SAME_WORK=${SAME_WORK:-$PWD/work}

# fixup VO name
VO_NAME=${VO_NAME:-$PBS_O_QUEUE}
VO_NAME=${VO_NAME:-$PBS_QUEUE}
VO_NAME=${VO_NAME:-$GROUP}
sed "s/^vo=.*/vo=$VO_NAME/" ./client/etc/same.conf.EGEE > ./client/etc/same.conf

# guess CE name
CE_NAME=${CE_NAME:-$PBS_O_HOST}
CE_NAME=${CE_NAME:-$EDG_WL_LOG_DESTINATION}
CE_NAME=${CE_NAME:-$GLITE_WMS_LOG_DESTINATION}
CE_NAME=${CE_NAME:-$SITE_GIIS_URL}
CE_NAME=${CE_NAME:-`hostname -f`}

# guess CE type
if [ -z "$SENSOR" ]; then
    if [ "$GLOBUS_GRAM_JOB_CONTACT" ]; then
        SENSOR=testjob
    else
        SENSOR=glite-testjob
    fi
fi

# guess central SE
if [ -z "$CENTRAL_SE" ]; then
    AVAIL_SE=availSE.lst
    PREF_SE=./client/sensors/CE/prefSE.lst

    # get available SEs for this VO
    ldapsearch -H ldap://$LCG_GFAL_INFOSYS -x -b o=grid \
    "(&(objectclass=GlueSA)(GlueSAAccessControlBaseRule=$VO_NAME))" \
    GlueChunkKey | sed -n -e '/^GlueChunkKey.*=/s///p' | sort -u > $AVAIL_SE

    # don't use default SE as cental
    VO=`echo $VO_NAME | tr '[a-z].-' '[A-Z]__'`
    DEFAULT_SE=`eval echo \\\$VO_${VO}_DEFAULT_SE`
    if [ "$DEFAULT_SE" ]; then
        grep -v $DEFAULT_SE $AVAIL_SE > $AVAIL_SE.new
        mv $AVAIL_SE.new $AVAIL_SE
    fi

    # select a preferred one
    for SE in `cat $AVAIL_SE`; do
        if grep -q -s $SE $PREF_SE; then
            CENTRAL_SE=$SE
            echo Using preferred SE: $CENTRAL_SE
            break
        fi
    done

    # or the first one if no preferred
    if [ -z "$CENTRAL_SE" -a -s $AVAIL_SE ]; then
        CENTRAL_SE=`head -1 $AVAIL_SE`
        echo Using first available SE: $CENTRAL_SE
    fi

    # or fallback to a known one
    if [ -z "$CENTRAL_SE" ]; then
        CENTRAL_SE=srm.cern.ch
        echo Using default SE: $CENTRAL_SE
    fi
fi

# dummy
ENV=${ENV:-ssam}

# run the tests
set -x
./client/bin/same-exec --nodetest $SENSOR $CE_NAME -- $ENV $CENTRAL_SE

# vim:et:ts=4:sw=4
